import asyncio
import datetime
import json
import time

# https://git.gurtam.net/labs/internal/gmqtt
import gmqtt

import uvloop

asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())

loop = asyncio.get_event_loop()


async def run():
    # MQTT connection credentials
    mqtt_host = "mqtt.flespi.io"
    mqtt_port = 1883
    mqtt_token = "XXXXXXX"
    mqtt_subscribe_topic = "debug_stream/#"
    mqtt_lbs_topic = "hw/lbs"

    # Statistics dump interval
    interval = 10

    # Create mqtt client
    c = gmqtt.Client("gmqtt_test_uvloop")

    # See "About that loop..." below.

    c.set_auth_credentials(mqtt_token)

    # Connect to broker
    await c.connect(mqtt_host, mqtt_port)
    print("Connected to ", mqtt_host)

    # subscribe
    subscribed = asyncio.Event(loop=loop)

    def on_subscribe(client, mid, granted_qos):
        """Callback when the broker responds to a subscribe request

        """
        subscribed.set()

    c.on_subscribe = on_subscribe

    qos = 0
    c.subscribe(mqtt_subscribe_topic, qos=qos)
    await subscribed.wait()

    def on_message(client, print_topic, payload, qos, counter):
        """Called when a message has been received

        """
        try:
            data = json.loads(payload.decode('utf-8', errors='replace'))
            counter.incr_counter()

            # process message
            if 'id' in data:
                # split message to fields
                fields = data['msg']
                pos = fields['pos']
                p = fields['p']

                # check pos and lbs params in message
                if p and pos and 'mcc' in p and 'mnc' in p and 'lac' in p \
                        and 'cell_id' in p:
                    counter.incr_lbs_counter()
                    # send lbs message to specified topic
                    lbs_msg = json.dumps(dict(pos=pos, params=p))
                    # print(lbs_msg)
                    client.publish(mqtt_lbs_topic, bytearray(
                        lbs_msg.encode('utf-8')), qos=qos
                                                  )

        except Exception as e:
            print(e)
            pass

    class Counter:
        __slots__ = ["_msgs_counter", "_lbs_counter"]

        def __init__(self):
            self._msgs_counter = 0
            self._lbs_counter = 0

        def incr_counter(self):
            self._msgs_counter += 1

        def incr_lbs_counter(self):
            self._lbs_counter += 1

        def dump_counter(self, interval):
            print("{}: {} msgs/sec, lbs: {} msgs/sec".format(
                datetime.datetime.now(),
                self._msgs_counter / interval,
                self._lbs_counter / interval
            )
            )

            self._msgs_counter = 0
            self._lbs_counter = 0

    counter = Counter()

    c.on_message = lambda client, print_topic, payload, qos: on_message(client,
                                                                        print_topic,
                                                                        payload,
                                                                        qos,
                                                                        counter)

    while True:
        await asyncio.sleep(interval)
        counter.dump_counter(interval)


loop.run_until_complete(run())
